>From 5c4a2c308f084632be8bb5d6293fca92c10de3f8 Mon Sep 17 00:00:00 2001
From: Andres Freund <andres@anarazel.de>
Date: Wed, 5 Mar 2014 00:15:39 +0100
Subject: [PATCH 4/4] Documentation for logical decoding.

Craig Ringer, Andres Freund, Christian Kruse
---
 doc/src/sgml/catalogs.sgml            |  27 +-
 doc/src/sgml/changesetextraction.sgml | 586 ++++++++++++++++++++++++++++++++++
 doc/src/sgml/filelist.sgml            |   2 +
 doc/src/sgml/func.sgml                | 102 +++++-
 doc/src/sgml/postgres.sgml            |   1 +
 doc/src/sgml/protocol.sgml            |  95 +++++-
 doc/src/sgml/ref/alter_table.sgml     |   2 +-
 doc/src/sgml/ref/create_table.sgml    |  15 +-
 8 files changed, 808 insertions(+), 22 deletions(-)
 create mode 100644 doc/src/sgml/changesetextraction.sgml

diff --git a/doc/src/sgml/catalogs.sgml b/doc/src/sgml/catalogs.sgml
index 908f947..727a201 100644
--- a/doc/src/sgml/catalogs.sgml
+++ b/doc/src/sgml/catalogs.sgml
@@ -5177,7 +5177,7 @@
 
   <para>
    For more on replication slots,
-   see <xref linkend="streaming-replication-slots">.
+   see <xref linkend="streaming-replication-slots"> and <xref linkend="changesetextraction">.
   </para>
 
   <table>
@@ -5210,6 +5210,13 @@
      </row>
 
      <row>
+      <entry><structfield>plugin</structfield></entry>
+      <entry><type>text</type></entry>
+      <entry></entry>
+      <entry>The basename of the shared object containing the output plugin this logical slot is using, or null for physical slots.</entry>
+     </row>
+
+     <row>
       <entry><structfield>datoid</structfield></entry>
       <entry><type>oid</type></entry>
       <entry><literal><link linkend="catalog-pg-database"><structname>pg_database</structname></link>.oid</literal></entry>
@@ -5243,6 +5250,24 @@
      </row>
 
      <row>
+      <entry><structfield>xmin</structfield></entry>
+      <entry><type>xid</type></entry>
+      <entry></entry>
+      <entry>The oldest transaction that this slot needs the database to
+      retain.  <literal>VACUUM</literal> cannot remove catalog tuples deleted
+      by any later transaction.
+      </entry>
+     </row>
+
+     <row>
+      <entry><structfield>catalog_xmin</structfield></entry>
+      <entry><type>xid</type></entry>
+      <entry></entry>
+      <entry>The <literal>xmin</literal>, or oldest transaction ID, that this
+      slot forces to be retained in the system catalogs. </entry>
+     </row>
+
+     <row>
       <entry><structfield>restart_lsn</structfield></entry>
       <entry><type>pg_lsn</type></entry>
       <entry></entry>
diff --git a/doc/src/sgml/changesetextraction.sgml b/doc/src/sgml/changesetextraction.sgml
new file mode 100644
index 0000000..60b915c
--- /dev/null
+++ b/doc/src/sgml/changesetextraction.sgml
@@ -0,0 +1,586 @@
+<!-- doc/src/sgml/changesetextraction.sgml -->
+ <chapter id="changesetextraction">
+  <title>Changeset Extraction</title>
+  <indexterm zone="changesetextraction">
+   <primary>Changeset Extraction</primary>
+  </indexterm>
+  <para>
+   PostgreSQL provides infrastructure to stream the modifications performed
+   via SQL to external consumers which can be used to implement replication
+   solutions, perform auditing and similar tasks.
+  </para>
+
+  <para>
+   <indexterm><primary>Changeset Extraction Slot</primary></indexterm>
+   A changeset extraction slot ("logical replication slot") is a persistent
+   server-side record of the replay progress of a stream of changes. A stream of
+   changes is read from the slot to a receiving client program.
+  </para>
+  <para>
+   The format in which those changes are streamed is determined by the output
+   plugin used. While an example plugin is provided, additional plugins can be
+   written to extend the choice of available formats without modifying any
+   core code.
+   Every output plugin has access to each individual new row produced
+   by <command>INSERT</command>, and the new row version created
+   by <command>UPDATE</command>. Which data is available for rows affected
+   by <command>DELETE</command> and for the old row version in
+   an <command>UPDATE</command> depends on the relation's
+   configured <link linkend="SQL-CREATETABLE-REPLICA-IDENTITY"><literal>REPLICA
+   IDENTITY</literal></link>.
+  </para>
+  <para>
+   Changes can be consumed either using one streaming replication protocol
+   (see <xref linkend="protocol-replication"> and
+   <xref linkend="changesetextraction-walsender">), or by calling functions
+   via SQL (see <xref linkend="changesetextraction-sql">). It is also possible
+   to write additional methods of consuming the output of a replication slot
+   without modifying core code
+   (see <xref linkend="changesetextraction-writer">).
+  </para>
+
+  <sect1 id="changesetextraction-example">
+   <title>Changeset Extraction Example</title>
+   <para>
+    The following example shows usage of the SQL interface.
+   </para>
+   <para>
+    Before you can use changeset extraction you must edit
+    <literal>postgresql.conf</literal> and ensure the following parameters are
+    set to at least:
+    <programlisting>
+wal_level = logical
+max_replication_slots = 1
+max_wal_senders = 1
+    </programlisting>
+    and restart PostgreSQL. Then connect to the target database (in the example
+    below, <literal>postgres</literal>) as a superuser.
+   </para>
+   <programlisting>
+postgres=# -- max_replication_slots must be nonzero and wal_level must be logical
+postgres=# SHOW max_replication_slots;
+ max_replication_slots
+-----------------------
+ 1
+(1 row)
+
+postgres=# SHOW wal_level;
+ wal_level
+-----------
+ logical
+(1 row)
+
+postgres=# SELECT * FROM pg_replication_slots;
+ slot_name | plugin | slot_type | datoid | database | active | xmin | catalog_xmin | restart_lsn
+-----------+--------+-----------+--------+----------+--------+------+--------------+-------------
+(0 rows)
+
+postgres=# -- Create a slot named 'regression_slot' using the output plugin 'test_decoding'
+postgres=# SELECT * FROM pg_create_logical_replication_slot('regression_slot', 'test_decoding');
+    slotname     | xlog_position
+-----------------+---------------
+ regression_slot | 0/16B1970
+(1 row)
+
+postgres=# SELECT * FROM pg_replication_slots;
+    slot_name    |    plugin     | slot_type | datoid | database | active |  xmin  | catalog_xmin | restart_lsn
+-----------------+---------------+-----------+--------+----------+--------+--------+--------------+-------------
+ regression_slot | test_decoding | logical   |  12052 | postgres | f      |        |          684 | 0/16A4408
+(1 row)
+
+postgres=# -- There are no changes to see yet
+postgres=# SELECT * FROM pg_logical_slot_get_changes('regression_slot', NULL, NULL);
+ location | xid | data
+----------+-----+------
+(0 rows)
+
+postgres=# CREATE TABLE data(id serial primary key, data text);
+CREATE TABLE
+
+postgres=# -- DDL isn't replicated, so all you'll see is the transaction
+postgres=# SELECT * FROM pg_logical_slot_get_changes('regression_slot', NULL, NULL);
+ location  | xid |    data
+-----------+-----+------------
+ 0/16D5D48 | 688 | BEGIN 688
+ 0/16E0380 | 688 | COMMIT 688
+(2 rows)
+
+postgres=# -- Once changes are read, they're consumed and not emitted
+postgres=# -- in a subsequent call:
+postgres=# SELECT * FROM pg_logical_slot_get_changes('regression_slot', NULL, NULL);
+ location | xid | data
+----------+-----+------
+(0 rows)
+
+postgres=# BEGIN;
+postgres=# INSERT INTO data(data) VALUES('1');
+postgres=# INSERT INTO data(data) VALUES('2');
+postgres=# COMMIT;
+
+postgres=# SELECT * FROM pg_logical_slot_get_changes('regression_slot', NULL, NULL);
+ location  | xid |                     data
+-----------+-----+-----------------------------------------------
+ 0/16E0478 | 689 | BEGIN 689
+ 0/16E0478 | 689 | table public.data: INSERT: id[int4]:1 data[text]:'1'
+ 0/16E0580 | 689 | table public.data: INSERT: id[int4]:2 data[text]:'2'
+ 0/16E0650 | 689 | COMMIT 689
+(4 rows)
+
+postgres=# INSERT INTO data(data) VALUES('3');
+
+postgres=# -- You can also peek ahead in the change stream without consuming changes
+postgres=# SELECT * FROM pg_logical_slot_peek_changes('regression_slot', NULL, NULL);
+ location  | xid |                     data
+-----------+-----+-----------------------------------------------
+ 0/16E09C0 | 690 | BEGIN 690
+ 0/16E09C0 | 690 | table public.data: INSERT: id[int4]:3 data[text]:'3'
+ 0/16E0B90 | 690 | COMMIT 690
+(3 rows)
+
+postgres=# -- You can also peek ahead in the change stream without consuming changes
+postgres=# SELECT * FROM pg_logical_slot_peek_changes('regression_slot', NULL, NULL);
+ location  | xid |                     data
+-----------+-----+-----------------------------------------------
+ 0/16E09C0 | 690 | BEGIN 690
+ 0/16E09C0 | 690 | table public.data: INSERT: id[int4]:3 data[text]:'3'
+ 0/16E0B90 | 690 | COMMIT 690
+(3 rows)
+
+postgres=# -- options can be passed to output plugin, to influence the formatting
+postgres=# SELECT * FROM pg_logical_slot_peek_changes('regression_slot', NULL, NULL, 'include-timestamp', 'on');
+ location  | xid |                     data
+-----------+-----+-----------------------------------------------
+ 0/16E09C0 | 690 | BEGIN 690
+ 0/16E09C0 | 690 | table public.data: INSERT: id[int4]:3 data[text]:'3'
+ 0/16E0B90 | 690 | COMMIT 690 (at 2014-02-27 16:41:51.863092+01)
+(3 rows)
+
+postgres=# -- Remember to destroy a slot you no longer need to stop it consuming
+postgres=# -- server resources:
+postgres=# SELECT pg_drop_replication_slot('regression_slot');
+ pg_drop_replication_slot
+-----------------------
+
+(1 row)
+    </programlisting>
+   <para>
+    The following example shows usage of the walsender interface using
+    the <link linkend="app-pgrecvlogical"><command>pg_recvlogical</command></link>
+    shell command. It requires the replication configurations to be allowed
+    (see <xref linkend="streaming-replication-authentication">)
+    and <varname>max_wal_senders</varname> to be set sufficiently high for
+    another connection.
+   </para>
+   <programlisting>
+# pg_recvlogical -d testdb --slot test --create
+# pg_recvlogical -d testdb --slot test --start -f -
+CTRL-Z
+# psql -c "INSERT INTO data(data) VALUES('4');"
+# fg
+BEGIN 693
+table public.data: INSERT: id[int4]:4 data[text]:'4'
+COMMIT 693
+CTRL-C
+# pg_recvlogical -d testdb --slot test --drop
+   </programlisting>
+  </sect1>
+  <sect1 id="changesetextraction-explanation">
+   <title>Changeset Extraction Concepts</title>
+   <sect2>
+    <indexterm>
+     <primary>Changeset Extraction</primary>
+    </indexterm>
+    <indexterm>
+     <primary>Changeset Generation</primary>
+    </indexterm>
+    <title>Changeset Extraction</title>
+    <para>
+     Changeset Extraction is the the process of extracting all persistent
+     changes to a database's tables into a coherent, easy to understand format
+     which can be interpreted without detailed knowledge of the database's
+     internal state.
+    </para>
+    <para>
+     In <productname>PostgreSQL</productname> changeset extraction is
+     implemented by decoding the <link linkend="wal">WAL's</link> contents,
+     which describe changes on a storage level, into an application-specific
+     form such as a stream of tuples or SQL statements.
+    </para>
+   </sect2>
+
+   <sect2>
+    <indexterm>
+     <primary>Logical Replication Slot</primary>
+    </indexterm>
+    <indexterm>
+     <primary>Replication Slot</primary>
+    </indexterm>
+    <title>Replication Slots</title>
+    <para>
+     In the context of changeset extraction a replication slot represents a
+     stream of changes which can be replayed to a client in the order they
+     were made on the origin server. Each slot streams a sequence of changes
+     from a single database, sending each change exactly once (unless peeking
+     forward in the stream).
+    </para>
+    <note>
+     <para>PostgreSQL also has streaming replication slots
+     (see <xref linkend="streaming-replication">), but they are used somewhat
+     differently there.
+     </para>
+    </note>
+    <para>
+     Replication slots have an identifier which is unique across all databases
+     in a <productname>PostgreSQL</productname> cluster. Slots persist
+     independently of the connection using them and are crash-safe.  They can
+     be allocated in primary servers and hot-standby servers (streaming
+     replicas or archive-replaying hot standbys), so it's possible to
+     replicate changes over physical replication then read a logical change
+     stream from a physical replica server.
+    </para>
+    <para>
+     Multiple independent slots may exist for a single database. Each slot has
+     its own state, allowing different consumers to receive changes from
+     different points in the database change stream. For most applications a
+     separate slot is required for each changeset consumer.
+    </para>
+    <para>
+     A changeset extraction slot knows nothing about the state of the
+     receiver(s).  It's even possible to have multiple different receivers use
+     the same slot at different times; they'll just get the changes following
+     on from when the last receiver stopped consuming them. Only one receiver
+     may consume changes from a slot at any given time.
+    </para>
+    <sect3 id="changesetextraction-abandoned-slots">
+     <title>Unused/abandoned slots</title>
+     <para>
+      Changeset extraction slots persist across crashes and know nothing about
+      the state of their consumer(s). They will prevent removal of required
+      resources even when there is no connection using them. This consumes
+      storage because neither required WAL nor required rows from the system
+      catalogs can be removed by VACUUM as long as they are required by a
+      replication slot, so if a slot is no longer required it should be
+      dropped.
+     </para>
+     <para>
+      Slots may be created manually, or using a client like a replication
+      tool. Keep this in mind when you retire a client that uses a changeset
+      extraction slot, like a logical replica - you may need to give it a
+      specific command to remove its changeset extraction slot, or drop the
+      slot yourself, even if you did not initially create the slot by hand.
+     </para>
+    </sect3>
+   </sect2>
+   <sect2>
+    <title>Exported Snapshots</title>
+    <para>
+     When a new replication slot is created over the walsender interface a
+     snapshot is exported
+     (see <xref linkend="functions-snapshot-synchronization">) which will show
+     exactly the state of the database after which all changes will be
+     included in the changestream. This can be used to create a new replica by
+     using <link linkend="sql-set-transaction"><literal>SET TRANSACTION
+     SNAPSHOT</literal></link> to read the state of the database at the moment
+     the slot was created. This transaction can then be used to dump the
+     database's state at that point in time which afterwards can be updated
+     using the slot's contents without loosing any changes.
+    </para>
+   </sect2>
+  </sect1>
+  <sect1 id="changesetextraction-walsender">
+   <title>Streaming Replication Protocol Interface</title>
+   <para>
+    The <literal>CREATE_REPLICATION_SLOT SLOT slotname LOGICAL
+    options</literal>, <literal>DROP_REPLICATION_SLOT SLOT slotname</literal>
+    and <literal>START_REPLICATION SLOT slotname LOGICAL options</literal>
+    commands can be used to create, drop and stream changes from a replication
+    slot respectively. These commands are only available over a replication
+    connection; the won't work from the SQL
+    level. See <xref linkend="protocol-replication">.
+   </para>
+   <para>
+    The <command>pg_recvlogical</command> command
+    (see <xref linkend="app-pgrecvlogical">) can be used to control changeset
+    extraction over a walsender connection.
+   </para>
+  </sect1>
+  <sect1 id="changesetextraction-sql">
+   <title>Changeset Extraction <acronym>SQL</acronym> Interface</title>
+   <para>
+     See <xref linkend="functions-replication"> for detailed
+     documentation on the SQL-level API for interacting with changeset
+     extraction.
+   </para>
+   <para>
+    Synchronous replication (see <xref linkend="synchronous-replication">) is
+    only supported on replication slots used over the walsender interface. The
+    function interface and additional, non-core, interfaces, do not support
+    synchronous replication.
+   </para>
+  </sect1>
+  <sect1 id="changesetextraction-catalogs">
+   <title>System catalogs related to changeset extraction</title>
+   <para>
+    The <xref linkend="catalog-pg-replication-slots"> view and the
+    <literal>pg_stat_replication</literal> view
+    in <xref linkend="monitoring-stats-views-table"> provide information about
+    the current state of replication slots and walsender connections
+    respectively. These views apply to both physical and logical replication.
+   </para>
+  </sect1>
+  <sect1 id="changesetextraction-output-plugin">
+   <title>Changeset Extraction Output Plugins</title>
+   <para>
+    An example output plugin can be found in the
+    <link linkend="test-decoding">
+     <filename>contrib/test_decoding</filename>
+    </link>
+    subdirectory of the PostgreSQL source tree.
+   </para>
+   <sect2 id="changesetextraction-output-init">
+    <title>Initialization Function</title>
+    <indexterm zone="changesetextraction">
+     <primary>_PG_output_plugin_init</primary>
+    </indexterm>
+    <para>
+     An output plugin is loaded by dynamically loading a shared library with
+     the output plugin's name as the library basename. The normal library
+     search path is used to locate the library. To provide the required output
+     plugin callbacks and to indicate that the library is actually an output
+     plugin it needs to provide a function named
+     <function>_PG_output_plugin_init</function>. This function is passed a
+     struct that needs to be filled with the callback function pointers for
+     individual actions.
+     <programlisting>
+typedef struct OutputPluginCallbacks
+{
+    LogicalDecodeStartupCB startup_cb;
+    LogicalDecodeBeginCB begin_cb;
+    LogicalDecodeChangeCB change_cb;
+    LogicalDecodeCommitCB commit_cb;
+    LogicalDecodeShutdownCB shutdown_cb;
+} OutputPluginCallbacks;
+typedef void (*LogicalOutputPluginInit)(struct OutputPluginCallbacks *cb);
+     </programlisting>
+     The <function>begin_cb</function>, <function>change_cb</function>
+     and <function>commit_cb</function> callbacks are required,
+     while <function>startup_cb</function>
+     and <function>shutdown_cb</function> are optional.
+    </para>
+   </sect2>
+
+   <sect2 id="changesetextraction-capabilities">
+    <title>Capabilities</title>
+    <para>
+     To decode, format and output changes, output plugins can use most of the
+     backend's normal infrastructure, including calling output functions. Read
+     only access to relations is permitted as long as only relations are
+     accessed that either have been created by initdb in
+     the <literal>pg_catalog</literal> schema, or have are marked as user
+     provided catalog tables using
+     <programlisting>
+ALTER TABLE user_catalog_table SET (user_catalog_table = true);
+CREATE TABLE another_catalog_table(data text) WITH (user_catalog_table = true);
+     </programlisting>
+     Any actions leading to XID assignment are prohibited. That, among others,
+     includes writing to tables, performing DDL changes and
+     calling <literal>txid_current()</literal>.
+    </para>
+   </sect2>
+
+   <sect2 id="changesetextraction-output-plugin-callbacks">
+    <title>Output Plugin Callbacks</title>
+    <para>
+     An output plugin gets notified about changes that are happening via
+     various callbacks it needs to provide.
+    </para>
+    <para>
+     Concurrent transactions are decoded in commit order and only changes
+     belonging to a specific transaction are decoded inbetween
+     the <literal>begin</literal> and <literal>commit</literal>
+     callbacks. Transaction that were rolled back explicitly or implicitly
+     will never be
+     decoded. Successfull <link linkend="SQL-SAVEPOINT">SAVEPOINTs</link> are
+     folded into the transaction containing them in the order they were
+     exectuded within that transaction.
+    </para>
+    <note>
+     <para>
+      Only transactions that have already safely been flushed to disk will be
+      decoded. That can lead to a COMMIT not immediately being decoded in a
+      directly following <literal>pg_logical_slot_get_changes()</literal>
+      when <varname>synchronous_commit</varname> is set
+      to <literal>off</literal>.
+     </para>
+    </note>
+    <sect3 id="changesetextraction-output-plugin-startup">
+     <title>Startup Callback</title>
+     <para>
+      The optional <function>startup_cb</function> callback is called whenever
+      an replication slot is created or asked to stream changes, independent
+      of the number of changes that are ready to be output.
+      <programlisting>
+typedef void (*LogicalDecodeStartupCB) (
+    struct LogicalDecodingContext *ctx,
+    OutputPluginOptions *options,
+    bool is_init
+);
+      </programlisting>
+      The <literal>is_init</literal> paramter will be true when the
+      replication slot is being created and false
+      otherwise. <parameter>options</parameter> points to a struct of options
+      that output plugins can set:
+      <programlisting>
+typedef struct OutputPluginOptions
+{
+    OutputPluginOutputType output_type;
+} OutputPluginOptions;
+      </programlisting>
+      <literal>output_type</literal> has to either be set to
+      <literal>OUTPUT_PLUGIN_TEXTUAL_OUTPUT</literal>
+      or <literal>OUTPUT_PLUGIN_BINARY_OUTPUT</literal>.
+     </para>
+     <para>
+      The startup callback should validate the options present in
+      <literal>ctx-&gt;output_plugin_options</literal>. If the output plugin
+      needs to have a state, it can
+      use <literal>ctx-&gt;output_plugin_private</literal> to store it.
+     </para>
+    </sect3>
+    <sect3 id="changesetextraction-output-plugin-shutdown">
+     <title>Shutdown Callback</title>
+     <para>
+      The optional <function>shutdown_cb</function> callback is called
+      whenever a formerly active replication slot is not used anymore and can
+      be used to deallocate resources private to the output plugin. The slot
+      isn't necessarily being dropped, streaming is just being stopped.
+      <programlisting>
+typedef void (*LogicalDecodeShutdownCB) (
+    struct LogicalDecodingContext *ctx
+);
+      </programlisting>
+     </para>
+   </sect3>
+    <sect3 id="changesetextraction-output-plugin-begin">
+     <title>Transaction Begin Callback</title>
+     <para>
+      The required <function>begin_cb</function> callback is called whenever a
+      transaction start has been decoded, but only if we know that the
+      transaction has committed. Aborted transactions and their contents are
+      never decoded.
+      <programlisting>
+typedef void (*LogicalDecodeBeginCB) (
+    struct LogicalDecodingContext *,
+    ReorderBufferTXN *txn
+);
+      </programlisting>
+      The <parameter>txn</parameter> parameter contains meta information about
+      the transaction, like the timestamp at which it committed and its XID.
+     </para>
+   </sect3>
+    <sect3 id="changesetextraction-output-plugin-commit">
+     <title>Transaction End Callback</title>
+     <para>
+      The required <function>commit_cb</function> callback is called whenever
+      a transaction commit has been
+      decoded. The <function>change_cb</function> callbacks for all modified
+      rows will have been called before this, if there have been any modified
+      rows.
+      <programlisting>
+typedef void (*LogicalDecodeCommitCB) (
+    struct LogicalDecodingContext *,
+    ReorderBufferTXN *txn
+);
+      </programlisting>
+     </para>
+    </sect3>
+    <sect3 id="changesetextraction-output-plugin-change">
+     <title>Callback called for each individual change in a
+     transaction</title>
+     <para>
+      The required <function>change_cb</function> callback is called for every
+      individual row modification inside a transaction, may it be
+      an <command>INSERT</command>, <command>UPDATE</command>
+      or <command>DELETE</command>. Even if the original command modified
+      several rows at once the callback will be called indvidually for each
+      row.
+      <programlisting>
+typedef void (*LogicalDecodeChangeCB) (
+    struct LogicalDecodingContext *ctx,
+    ReorderBufferTXN *txn,
+    Relation relation,
+    ReorderBufferChange *change
+);
+      </programlisting>
+      The <parameter>ctx</parameter> and <parameter>txn</parameter> parameters
+      have the same contents as for the <function>begin_cb</function>
+      and <function>commit_cb</function> callbacks, but additionally the
+      relation descriptor <parameter>relation</parameter> for the relation the
+      row belongs to and a struct
+      <parameter>change</parameter> describing the row modification are passed
+      in.
+     </para>
+     <note>
+      <para>
+       Only changes in user defined tables that are not unlogged
+       (see <xref linkend="SQL-CREATETABLE-UNLOGGED">) and not temporary
+       (see <xref linkend="SQL-CREATETABLE-TEMPORARY">) can be extracted using
+       changeset extraction.
+      </para>
+     </note>
+    </sect3>
+   </sect2>
+   <sect2 id="changesetextraction-output-plugin-output">
+    <title>Functions for producing output from an output plugin</title>
+    <para>
+     To actually produce output, output plugins can write data to
+     the <literal>StringInfo</literal> output buffer
+     in <literal>ctx-&gt;out</literal> when inside
+     the <function>begin_cb</function>, <function>commit_cb</function>
+     or <function>change_cb</function> callbacks. Before writing to the output
+     buffer <function>OutputPluginPrepareWrite(ctx, last_write)</function> has
+     to be called, and after finishing writing to the
+     buffer <function>OutputPluginWrite(ctx, last_write)</function> has to be
+     called to perform the write. The <parameter>last_write</parameter>
+     indicates whether a particular write was the callback's last write.
+    </para>
+    <para>
+     The following example shows how to output data to the consumer of an
+     output plugin:
+     <programlisting>
+OutputPluginPrepareWrite(ctx, true);
+appendStringInfo(ctx->out, "BEGIN %u", txn->xid);
+OutputPluginWrite(ctx, true);
+     </programlisting>
+    </para>
+   </sect2>
+  </sect1>
+  <sect1 id="changesetextraction-writer">
+   <title>Changeset Extraction Output Writers</title>
+   <para>
+    It is possible to add more output methods in addition to the SQL
+    and replication protocol variants for consuming changeset extraction
+    data. For details look at the implementation of the SQL interface
+    functions
+    in <filename>src/backend/replication/logical/logicalfuncs.c</filename>.
+    Essentially three functions need to be provided, one to read WAL, one to
+    prepare writing output and one to write the output
+    (see <xref linkend="changesetextraction-output-plugin-output">).
+   </para>
+  </sect1>
+  <sect1 id="changesetextraction-synchronous">
+   <title>Synchronous replication support for Changeset Extraction</title>
+   <para>
+    The Changeset Extraction support in <productname>PostgreSQL</productname>
+    may be used to to
+    build <link linkend="synchronous-replication">synchronous
+    replication</link> solutions with the same user interface as synchronous
+    replication for <link linkend="streaming-replication">streaming
+    replication</link> if the walsender interface
+    (see <xref linkend="changesetextraction-walsender">) is used to stream out
+    data. Clients have to send <literal>Standby status update (F)</literal>
+    (see <xref linkend="protocol-replication">), just like streaming
+    replication clients do.
+   </para>
+  </sect1>
+ </chapter>
diff --git a/doc/src/sgml/filelist.sgml b/doc/src/sgml/filelist.sgml
index 0e863ee..0208ca2 100644
--- a/doc/src/sgml/filelist.sgml
+++ b/doc/src/sgml/filelist.sgml
@@ -91,6 +91,7 @@
 <!ENTITY nls        SYSTEM "nls.sgml">
 <!ENTITY plhandler  SYSTEM "plhandler.sgml">
 <!ENTITY fdwhandler SYSTEM "fdwhandler.sgml">
+<!ENTITY changesetextraction SYSTEM "changesetextraction.sgml">
 <!ENTITY protocol   SYSTEM "protocol.sgml">
 <!ENTITY sources    SYSTEM "sources.sgml">
 <!ENTITY storage    SYSTEM "storage.sgml">
@@ -146,6 +147,7 @@
 <!ENTITY test-decoding   SYSTEM "test-decoding.sgml">
 <!ENTITY test-parser     SYSTEM "test-parser.sgml">
 <!ENTITY test-shm-mq     SYSTEM "test-shm-mq.sgml">
+<!ENTITY test-decoding   SYSTEM "test-decoding.sgml">
 <!ENTITY tsearch2        SYSTEM "tsearch2.sgml">
 <!ENTITY unaccent      SYSTEM "unaccent.sgml">
 <!ENTITY uuid-ossp       SYSTEM "uuid-ossp.sgml">
diff --git a/doc/src/sgml/func.sgml b/doc/src/sgml/func.sgml
index a1f627c..8a18602 100644
--- a/doc/src/sgml/func.sgml
+++ b/doc/src/sgml/func.sgml
@@ -16363,8 +16363,9 @@ postgres=# SELECT * FROM pg_xlogfile_name_offset(pg_stop_backup());
 
    <para>
     PostgreSQL exposes a number of functions for controlling and interacting
-    with replication features. See <xref linkend="streaming-replication">
-    and <xref linkend="streaming-replication-slots">.
+    with replication features. See <xref linkend="streaming-replication">,
+    <xref linkend="streaming-replication-slots">
+    and <xref linkend="changesetextraction">.
    </para>
 
    <para>
@@ -16423,9 +16424,106 @@ postgres=# SELECT * FROM pg_xlogfile_name_offset(pg_stop_backup());
         command <literal>DROP_REPLICATION_SLOT</>.
        </entry>
       </row>
+
+      <row>
+       <entry>
+        <indexterm>
+         <primary>pg_create_logical_replication_slot</primary>
+        </indexterm>
+        <literal><function>pg_create_logical_replication_slot(<parameter>slotname</parameter> <type>name</type>, <parameter>plugin</parameter> <type>name</type>)</function></literal>
+       </entry>
+       <entry>
+        (<parameter>slotname</parameter> <type>name</type>, <parameter>xlog_position</parameter> <type>pg_lsn</type>)
+       </entry>
+       <entry>
+        Creates a new logical (decoding) replication slot named
+        <parameter>slotname</parameter> using the output plugin
+        <parameter>plugin</parameter>. Output plugins are listed amongst the
+        extensions in <literal>pg_catalog.pg_available_extensions</literal>,
+        but there is no specific listing of only output plugins. Same as
+        walsender protocol command <literal>CREATE REPLICATION SLOT ... LOGICAL</literal>.
+       </entry>
+      </row>
+
+      <row>
+       <entry>
+        <indexterm>
+         <primary>pg_logical_slot_get_changes</primary>
+        </indexterm>
+        <literal><function>pg_logical_slot_get_changes(<parameter>slotname</parameter> <type>name</type>, <parameter>upto_lsn</parameter> <type>pg_lsn</type>, <parameter>upto_nchanges</parameter> <type>int</type>, VARIADIC <parameter>options</parameter> <type>text[]</type>)</function></literal>
+       </entry>
+       <entry>
+        (<parameter>location</parameter> <type>pg_lsn</type>, <parameter>xid</parameter> <type>xid</type>, <parameter>data</parameter> <type>text</type>)
+       </entry>
+       <entry>
+        Returns changes in the slot <parameter>slotname</parameter>, starting
+        from the point at which since changes have been consumed last. Changes
+        are extracted upto the current end of the WAL, or if nonnull only if
+        their transactions have committed
+        before <parameter>upto_lsn</parameter>, or if nonnull
+        until <parameter>upto_nchanges</parameter> rows have been
+        collected. Note that the <parameter>upto_nchanges</parameter> limit is
+        only enforced everytime a transaction commits. Changes will not be
+        consumed.
+       </entry>
+      </row>
+
+      <row>
+       <entry>
+        <indexterm>
+         <primary>pg_logical_slot_peek_changes</primary>
+        </indexterm>
+        <literal><function>pg_logical_slot_peek_changes(<parameter>slotname</parameter> <type>name</type>, <parameter>upto_lsn</parameter> <type>pg_lsn</type>, <parameter>upto_nchanges</parameter> <type>int</type>, VARIADIC <parameter>options</parameter> <type>text[]</type>)</function></literal>
+       </entry>
+       <entry>
+        (<parameter>location</parameter> <type>text</type>, <parameter>xid</parameter> <type>xid</type>, <parameter>data</parameter> <type>text</type>)
+       </entry>
+       <entry>
+        Behaves just like
+        the <function>pg_logical_slot_get_changes()</function> function,
+        except that changes are not consumed, i.e. will be returned again on
+        future calls.
+       </entry>
+      </row>
+
+      <row>
+       <entry>
+        <indexterm>
+         <primary>pg_logical_slot_get_binary_changes</primary>
+        </indexterm>
+        <literal><function>pg_logical_slot_get_binary_changes(<parameter>slotname</parameter> <type>name</type>, <parameter>upto_lsn</parameter> <type>pg_lsn</type>, <parameter>upto_nchanges</parameter> <type>int</type>, VARIADIC <parameter>options</parameter> <type>text[]</type>)</function></literal>
+       </entry>
+       <entry>
+        (<parameter>location</parameter> <type>pg_lsn</type>, <parameter>xid</parameter> <type>xid</type>, <parameter>data</parameter> <type>bytea</type>)
+       </entry>
+       <entry>
+        Behaves just like
+        the <function>pg_logical_slot_get_changes()</function> function,
+        except that changes are returned as <type>bytea</type>.
+       </entry>
+      </row>
+
+      <row>
+       <entry>
+        <indexterm>
+         <primary>pg_logical_slot_peek_binary_changes</primary>
+        </indexterm>
+        <literal><function>pg_logical_slot_peek_binary_changes(<parameter>slotname</parameter> <type>name</type>, <parameter>upto_lsn</parameter> <type>pg_lsn</type>, <parameter>upto_nchanges</parameter> <type>int</type>, VARIADIC <parameter>options</parameter> <type>text[]</type>)</function></literal>
+       </entry>
+       <entry>
+        (<parameter>location</parameter> <type>pg_lsn</type>, <parameter>xid</parameter> <type>xid</type>, <parameter>data</parameter> <type>bytea</type>)
+       </entry>
+       <entry>
+        Behaves just like
+        the <function>pg_logical_slot_get_changes()</function> function,
+        except that changes are returned as <type>bytea</type> and that
+        changes are not consumed, i.e. will be returned again on future calls.
+       </entry>
+      </row>
      </tbody>
     </tgroup>
    </table>
+
   </sect2>
 
   <sect2 id="functions-admin-dbobject">
diff --git a/doc/src/sgml/postgres.sgml b/doc/src/sgml/postgres.sgml
index b47bf52..dd1709f 100644
--- a/doc/src/sgml/postgres.sgml
+++ b/doc/src/sgml/postgres.sgml
@@ -219,6 +219,7 @@
 
   &spi;
   &bgworker;
+  &changesetextraction;
 
  </part>
 
diff --git a/doc/src/sgml/protocol.sgml b/doc/src/sgml/protocol.sgml
index 510bf9a..3621772 100644
--- a/doc/src/sgml/protocol.sgml
+++ b/doc/src/sgml/protocol.sgml
@@ -1301,24 +1301,48 @@
 <title>Streaming Replication Protocol</title>
 
 <para>
-To initiate streaming replication, the frontend sends the
-<literal>replication</> parameter in the startup message. A boolean value
-of <literal>true</> tells the backend to go into walsender mode, wherein a
-small set of replication commands can be issued instead of SQL statements. Only
-the simple query protocol can be used in walsender mode.
-Passing a <literal>database</> as the value instructs walsender to connect to
-the database specified in the <literal>dbname</> paramter which will in future
-allow some additional commands to the ones specified below to be run.
+ To initiate streaming replication, the frontend sends the
+ <literal>replication</> parameter in the startup message. This accepts the
+ values <literal>true</literal>, <literal>false</literal> (and their 1/0
+ numeric equivalents) or in 9.4 and above, <literal>database</literal>.
+</para>
+<para>
+ Any valid non-false value tells the backend to go into walsender mode, wherein
+ the small set of replication commands documented below can be issued instead
+ of SQL statements.  Only the simple query protocol can be used in walsender
+ mode.
+</para>
+<para>
+ If <literal>replication</literal> is <literal>true</literal>, no specific
+ database is connected to. In this mode only a subset of walsender subcommands
+ are available. To use database-specific walsender commands like the logical
+ replication commands, <literal>replication</> must be set to
+ <literal>database</> and the name of a database to connect to must be
+ specified in the <literal>database</> startup packet field.
+</para>
+<para>
+ For the purpose of testing replication commands, you can make a replication
+ connection via <application>psql</application> or any other <literal>libpq</literal>-using
+ tool with a connection string including the <literal>replication</literal> option,
+ e.g.:
+ <programlisting>
+  psql "dbname=postgres replication=database" -c "IDENTIFY_SYSTEM;"
+ </programlisting>
+ However it is usually more useful to use
+ <application>pg_receivexlog</application> (for physical replication) or
+ <application>pg_recvlogical</application> (for logical replication).
+</para>
 
+<para>
 The commands accepted in walsender mode are:
-
 <variablelist>
   <varlistentry>
     <term>IDENTIFY_SYSTEM</term>
     <listitem>
      <para>
-      Requests the server to identify itself. Server replies with a result
-      set of a single row, containing four fields:
+      Requests the server to identify itself. Does not require a database name
+      to be specified for the connection. Server replies with a result set of a
+      single row, containing four fields:
      </para>
 
      <para>
@@ -1381,7 +1405,8 @@ The commands accepted in walsender mode are:
     <listitem>
      <para>
       Requests the server to send over the timeline history file for timeline
-      <replaceable class="parameter">tli</replaceable>.  Server replies with a
+      <replaceable class="parameter">tli</replaceable>. Does not require a database
+      name to be specified for the connection. Server replies with a
       result set of a single row, containing two fields:
      </para>
 
@@ -1764,17 +1789,55 @@ The commands accepted in walsender mode are:
      </para>
     </listitem>
   </varlistentry>
+  <varlistentry>
+    <term><literal>START_REPLICATION</literal> <literal>SLOT</literal> <replaceable class="parameter">slotname</> <literal>LOGICAL</literal> <replaceable class="parameter">XXX/XXX</></term>
+    <listitem>
+     <para>
+      Instructs server to start streaming WAL for logical replication, starting
+      at WAL position <replaceable class="parameter">XXX/XXX</>. The server can
+      reply with an error, e.g. if the requested section of WAL has already
+      been recycled. On success, server responds with a CopyBothResponse
+      message, and then starts to stream WAL to the frontend.
+     </para>
+     <para>
+      The output plugin associated with the selected slot is used
+      to process the output for streaming.
+     </para>
+     <variablelist>
+      <varlistentry>
+       <term><literal>SLOT</literal> <replaceable class="parameter">slotname</></term>
+       <listitem>
+         <para>
+          The name of the slot to stream changes from. This parameter is required,
+          and must correspond to an existing logical replication slot created
+          with <literal>CREATE_REPLICATION_SLOT</literal> in
+          <literal>LOGICAL</literal> mode.
+         </para>
+       </listitem>
+      </varlistentry>
+      <varlistentry>
+       <term><replaceable class="parameter">XXX/XXX</></term>
+       <listitem>
+        <para>
+         The WAL position to begin streaming at.
+        </para>
+       </listitem>
+      </varlistentry>
+     </variablelist>
+    </listitem>
+  </varlistentry>
 
   <varlistentry>
-    <term><literal>DROP_REPLICATION_SLOT</literal> <replaceable class="parameter">slotname</></term>
+    <term><literal>DROP_REPLICATION_SLOT</literal> <literal>SLOT</literal> <replaceable class="parameter">slotname</></term>
     <listitem>
      <para>
-      Drops a replication slot, freeing any reserved server-side resources. If
-      the slot is currently in use by an active connection, this command fails.
+     Drops a physical or logical replication slot, freeing any reserved server-side
+     resources. If the slot is currently in use by an active connection this command
+     fails.
      </para>
      <variablelist>
       <varlistentry>
-       <term><replaceable class="parameter">slotname</></term>
+       <term><literal>SLOT</literal> <replaceable class="parameter">slotname</></term>
        <listitem>
          <para>
           The name of the slot to drop.
diff --git a/doc/src/sgml/ref/alter_table.sgml b/doc/src/sgml/ref/alter_table.sgml
index 2b02e66..4847d66 100644
--- a/doc/src/sgml/ref/alter_table.sgml
+++ b/doc/src/sgml/ref/alter_table.sgml
@@ -580,7 +580,7 @@ ALTER TABLE [ IF EXISTS ] <replaceable class="PARAMETER">name</replaceable>
     </listitem>
    </varlistentry>
 
-   <varlistentry>
+   <varlistentry id="SQL-CREATETABLE-REPLICA-IDENTITY">
     <term><literal>REPLICA IDENTITY</literal></term>
     <listitem>
      <para>
diff --git a/doc/src/sgml/ref/create_table.sgml b/doc/src/sgml/ref/create_table.sgml
index fc7ad09..f5a3e3e 100644
--- a/doc/src/sgml/ref/create_table.sgml
+++ b/doc/src/sgml/ref/create_table.sgml
@@ -137,7 +137,7 @@ CREATE [ [ GLOBAL | LOCAL ] { TEMPORARY | TEMP } | UNLOGGED ] TABLE [ IF NOT EXI
 
   <variablelist>
 
-   <varlistentry>
+   <varlistentry id="SQL-CREATETABLE-TEMPORARY">
     <term><literal>TEMPORARY</> or <literal>TEMP</></term>
     <listitem>
      <para>
@@ -171,7 +171,7 @@ CREATE [ [ GLOBAL | LOCAL ] { TEMPORARY | TEMP } | UNLOGGED ] TABLE [ IF NOT EXI
     </listitem>
    </varlistentry>
 
-   <varlistentry>
+   <varlistentry id="SQL-CREATETABLE-UNLOGGED">
     <term><literal>UNLOGGED</></term>
     <listitem>
      <para>
@@ -1051,6 +1051,17 @@ CREATE [ [ GLOBAL | LOCAL ] { TEMPORARY | TEMP } | UNLOGGED ] TABLE [ IF NOT EXI
     </listitem>
    </varlistentry>
 
+   <varlistentry>
+    <term><literal>user_catalog_table</literal> (<type>boolean</type>)</term>
+    <listitem>
+     <para>
+      Declare a table as an additional catalog table, e.g. for the purpose of
+      logical replication. See
+      <xref linkend="changesetextraction-capabilities"> for details.
+     </para>
+    </listitem>
+   </varlistentry>
+
    </variablelist>
 
   </refsect2>
-- 
1.8.3.251.g1462b67

